#!/usr/bin/env bash

set -e

run_command() {
  local cmd="$1"
  echo "Running $cmd as $(id -u)"
  if [[ "$ISROOT" == true ]]; then
    su -c "$cmd" forgejo-runner
  else
    eval "$cmd"
  fi
}

# Initial setup
if [[ ! -d /data ]]; then
  mkdir -p /data
fi
cd /data

RUNNER_USERID="${RUNNER_USERID:-1000}"

# Check if the script is running as root
if [[ $(id -u) -eq 0 ]]; then
  ISROOT=true
fi

if [[ "$ISROOT" == true ]]; then
  # Check if the forgejo-runner user exists
  if id "forgejo-runner" &>/dev/null; then
    echo "forgejo-runner user exists."

    # Change the user ID if needed
    CURRENT_UID=$(id -u forgejo-runner)
    if [[ "${CURRENT_UID}" -ne "${RUNNER_USERID}" ]]; then
      echo "Changing UID of forgejo-runner to ${RUNNER_USERID}"
      sed -i "s/^forgejo-runner:[^:]*:[^:]*:/forgejo-runner:x:${RUNNER_USERID}:/" /etc/passwd
    fi
  else
    echo "Creating user forgejo-runner with UID ${RUNNER_USERID}"
    adduser --uid "${RUNNER_USERID}" --home /home/forgejo-runner --disabled-password --gecos "" forgejo-runner
  fi

  # Ensure /data is owned by the runner user
  chown -R forgejo-runner:forgejo-runner /data
fi

# Handle and alter the config file
if [[ -z "${CONFIG_FILE}" ]]; then
  echo "CONFIG_FILE is not set"
  CONFIG_FILE="/data/config.yml"
fi
CONFIG_ARG="--config ${CONFIG_FILE}"

DOCKER_HOST=${DOCKER_HOST:-docker}
if [[ ! -f "${CONFIG_FILE}" ]]; then
  run_command "forgejo-runner generate-config > ${CONFIG_FILE}" forgejo-runner

  # Remove test environment variables if they exist in the config file
  sed -i "/^    A_TEST_ENV_NAME_1:/d" ${CONFIG_FILE}
  sed -i "/^    A_TEST_ENV_NAME_2:/d" ${CONFIG_FILE}

  # Apply default values for docker
  sed -i "/^  labels:/c\  \"labels\": [\"docker:docker://code.forgejo.org/oci/node:20-bookworm\", \"ubuntu-22.04:docker://catthehacker/ubuntu:act-22.04\"]" ${CONFIG_FILE}
  sed -i "/^  network:/c\  network: host" ${CONFIG_FILE}
  sed -i "/^  privileged:/c\  privileged: true" ${CONFIG_FILE}
  sed -i "/^  options:/c\  options: -v /certs/client:/certs/client" ${CONFIG_FILE}
  sed -i "/^  valid_volumes:/c\  valid_volumes:\n    - /certs/client" ${CONFIG_FILE}
  sed -i "/^  docker_host:/c\  docker_host: tcp://${DOCKER_HOST}:2376" ${CONFIG_FILE}
fi

if [[ ! -z "${ENV_FILE}" ]]; then
  sed -i "/^  env_file:/c\  env_file: ${ENV_FILE}" ${CONFIG_FILE}
else
  ENV_FILE="/data/.env"
fi

if [[ ! -f "${ENV_FILE}" ]]; then
  echo "Creating ${ENV_FILE} and populating with default values"
  cat <<EOF >${ENV_FILE}
  DOCKER_TLS_VERIFY: 1
  DOCKER_CERT_PATH: /certs/client
EOF
fi

EXTRA_ARGS=""
if [[ ! -z "${RUNNER_LABELS}" ]]; then
  EXTRA_ARGS="${EXTRA_ARGS} --labels ${RUNNER_LABELS}"
fi

# Set the runner file
if [[ -z "${RUNNER_FILE}" ]]; then
  RUNNER_FILE=".runner.json" # use json so editors know how to highlight
fi
sed -i "/^  file:/c\  file: ${RUNNER_FILE}" ${CONFIG_FILE}

if [[ ! -s "${RUNNER_FILE}" ]]; then
  touch ${RUNNER_FILE}
  try=$((try + 1))
  success=0

  if [[ ! -z "${FORGEJO_SECRET}" ]]; then
    EXTRA_ARGS="${EXTRA_ARGS} --secret ${FORGEJO_SECRET}"
  else
    if [[ -z "${RUNNER_TOKEN}" ]]; then
      echo "RUNNER_TOKEN is not set"
      exit 1
    fi
    EXTRA_ARGS="${EXTRA_ARGS} --token ${RUNNER_TOKEN}"
  fi

  # The point of this loop is to make it simple, when running both forgejo-runner and gitea in docker,
  # for the forgejo-runner to wait a moment for gitea to become available before erroring out.  Within
  # the context of a single docker-compose, something similar could be done via healthchecks, but
  # this is more flexible.
  while [[ $success -eq 0 ]] && [[ $try -lt ${MAX_REG_ATTEMPTS:-10} ]]; do
    # run_command "forgejo-runner register \
    #   --instance \"${FORGEJO_URL:-http://forgejo:3000}\" \
    #   --name \"${RUNNER_NAME:-$(hostname)}\" \
    #   ${CONFIG_ARG} ${EXTRA_ARGS} --no-interactive 2>&1 | tee /tmp/reg.log"

    run_command "forgejo-runner create-runner-file --connect \
    --instance \"${FORGEJO_URL:-http://forgejo:3000}\" \
    --name \"${RUNNER_NAME:-$(hostname)}\" \
    ${CONFIG_ARG} ${EXTRA_ARGS} 2>&1 | tee /tmp/reg.log"


    cat /tmp/reg.log | grep 'connection successful' >/dev/null
    if [[ $? -eq 0 ]]; then
      echo "SUCCESS"
      success=1
    else
      echo "Waiting to retry ..."
      sleep 5
    fi
  done
fi

# Prevent reading the token from the forgejo-runner process
unset RUNNER_TOKEN

run_command "forgejo-runner daemon ${CONFIG_ARG}"
